package it.polito.esame.gui;

import it.polito.esame.GraphModel;
import it.polito.esame.Model;
import it.polito.porto.bean.ArticleComparatorByDescendingYear;
import it.polito.porto.bean.PortoArticle;
import it.polito.porto.bean.PortoCreator;

import java.net.URL;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.ResourceBundle;
import java.util.Set;

import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;

public class Controller {

	private Model model;
	private GraphModel graphModel;

	@FXML
	// ResourceBundle that was given to the FXMLLoader
	private ResourceBundle resources;

	@FXML
	// URL location of the FXML file that was given to the FXMLLoader
	private URL location;

	@FXML
	// fx:id="btnCercaArticoli"
	private Button btnCercaArticoli; // Value injected by FXMLLoader

	@FXML
	// fx:id="txtResult"
	private TextArea txtResult; // Value injected by FXMLLoader

	@FXML
	// fx:id="txtIdCreator"
	private TextField txtIdCreator; // Value injected by FXMLLoader

	@FXML
	// fx:id="txtIdArticolo"
	private TextField txtIdArticolo; // Value injected by FXMLLoader

	@FXML
	// fx:id="btnCercaArticolo"
	private Button btnCercaArticolo; // Value injected by FXMLLoader

	@FXML
	void doCercaArticoli(ActionEvent event) {

		String matricola = txtIdCreator.getText();

		int creatorId;
		try {
			creatorId = Integer.parseInt(matricola);

			PortoCreator cre = model.getCreatorFromId(creatorId);

			if (cre == null) {
				txtResult.clear();
				txtResult
						.appendText("ERRORE: il numero di matricola specificato non esiste\n");

			} else {
				List<PortoArticle> list = cre.getArticoli();
				Collections.sort(list, new ArticleComparatorByDescendingYear());

				txtResult.clear();
				for (PortoArticle art : cre.getArticoli()) {
					txtResult.appendText(String.format("%4d %60.60s %d\n",
							art.getDate(), art.getTitle(), art.getEprintid()));
				}
			}

		} catch (NumberFormatException e) {
			txtResult.clear();
			txtResult
					.appendText("ERRORE: inserire un numero di matricola in formato numerico\n");
		}

	}

	@FXML
	void doCercaRevisori(ActionEvent event) {

		String idArticolo = txtIdArticolo.getText();

		try {
			long eprintId = Integer.parseInt(idArticolo);

			PortoArticle art = model.getArticleFromId(eprintId);

			if (art == null) {
				
				txtResult.clear();
				txtResult
						.appendText("ERRORE: il numero di articolo specificato non esiste\n") ;
			} else {

				/*
				 * L'algoritmo utilizzato  semplice: si compiono i primi 2
				 * passi di una VISITA IN AMPIEZZA del grafo, a partire dagli
				 * autori dell'articolo specificato. Al primo livello si
				 * troveranno tutti i co-autori degli autori originari. Al
				 * secondo livello si troveranon i co-autori dei co-autori.
				 * 
				 * Nessuno di questi potr essere un revisore, per cui alla fine
				 * calcoliamo la differenza insiemistica.
				 */

				// Livello 0: Trova gli autori dell'articolo
				List<PortoCreator> liv0 = art.getCreators();
				// printAuthors(0, liv0) ;

				// Livello 1: Trova i co-autori
				Set<PortoCreator> liv1 = graphModel.getCoAuthors(liv0);
				// printAuthors(1, liv1) ;

				// Livello 2: Trova i co-co-autori
				Set<PortoCreator> liv2 = graphModel.getCoAuthors(liv1);
				// printAuthors(2, liv2) ;

				// I revisori permessi sono TUTTI gli autori che NON sono in
				// liv0, liv1, n liv2
				Set<PortoCreator> revisori = new HashSet<PortoCreator>(
						model.getCreators());

				revisori.removeAll(liv0);
				revisori.removeAll(liv1);
				revisori.removeAll(liv2);

				// printAuthors(999, revisori) ;

				txtResult.clear();
				for (PortoCreator cre : revisori) {
					txtResult.appendText(String.format("%s %s\n",
							cre.getFamily(), cre.getGiven()));
				}
			}
		} catch (NumberFormatException e) {
			txtResult.clear();
			txtResult
					.appendText("ERRORE: inserire un numero di articolo in formato numerico\n");
		}

	}

/*	private void printAuthors(int liv, Collection<PortoCreator> list) {
		System.out.format("%d: ", liv);
		for (PortoCreator c : list)
			System.out.format("%s-%s ", c.getFamily(), c.getGiven());
		System.out.println();
	}
*/
	
	@FXML
	// This method is called by the FXMLLoader when initialization is complete
	void initialize() {
		assert btnCercaArticoli != null : "fx:id=\"btnCercaArticoli\" was not injected: check your FXML file 'peers.fxml'.";
		assert txtResult != null : "fx:id=\"txtResult\" was not injected: check your FXML file 'peers.fxml'.";
		assert txtIdCreator != null : "fx:id=\"txtIdCreator\" was not injected: check your FXML file 'peers.fxml'.";
		assert txtIdArticolo != null : "fx:id=\"txtIdArticolo\" was not injected: check your FXML file 'peers.fxml'.";
		assert btnCercaArticolo != null : "fx:id=\"btnCercaArticolo\" was not injected: check your FXML file 'peers.fxml'.";

	}

	public void setModel(Model model) {
		this.model = model;
	}

	public void setGraphModel(GraphModel graphModel) {
		this.graphModel = graphModel;
	}
}
